/*
  Datenformat:
  K;<ID Kiste>;<Lager-Nr/Fach Text><CR>
  K = Kiste
  Semikolon
  ID, siehe "Kiste-F7"
  Semikolon
  Text wie im Eingabefeld Lager-Nr/Fach
  CR = 0x0D

  oder

  SETM;<Alte ID>;<NEUE ID><CR>
  SETM = ID für Kiste ändern
  Semikolon
  Alte ID
  Semikolon
  Neue ID
  CR = 0x0D
*/

//#define debug
#include <EEPROM.h>

const byte numChars = 32;
char receivedChars[numChars];
char tempChars[numChars];        // temporary array for use when parsing
char messageFromPC[numChars] = {0};

int integerFromPC[4] = {0, 0, 0, 0};
int cathodes[11] = {13, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11}; //ROWS (first place is an Spacer, or start with 0 in EleLa)
int anodes[7] = {13, 14, 15, 16, 17, 18, 19}; // Culoms (first place is an Spacer, or start with 0 in EleLa) A0 alias Pin 14 through A5 alias Pin19
int _ID;
int ledState = LOW;             // ledState used to set the LED

boolean newData = false;
boolean waittime = false;
boolean first_start = false;

unsigned long currentMillis = 0;
unsigned long previousMillis = 0;        // will store last time LED was updated
const long interval = 25000;           // interval at which to blink (milliseconds)

//==================================================================================
void setup()
{
  Serial.begin(9600);
#ifdef debug
  Serial.println(" Datenformat: K;<ID Kiste>;<Lager-Nr/Fach Text><CR> ");
  Serial.println();
#endif
  //eeprom_write_action(999);
  if (EEPROM.read(0x03) == 255)
  {
    first_start = true;
  }

  if (first_start == true)
  {
    _ID = 999;
  }

  if (first_start == false)
  {
    _ID = eeprom_read_action();
  }

  for (int i = 0; i < 12; i++)
  {
    pinMode(cathodes[i], OUTPUT);
    digitalWrite (cathodes[i], LOW);
  }

  for (int i = 0; i < 7; i++)
  {
    pinMode(anodes[i], OUTPUT);
    digitalWrite (anodes[i], LOW);
  }
  #ifdef debug
  Serial.print("My ID is:");
  Serial.println(_ID);
  #endif
}

void loop() {
  currentMillis = millis();
  //Serial.println(currentMillis);
  if ( waittime == true)
  {
    if (currentMillis - previousMillis >= interval)
    {
      waittime = false;
      output_reset();
    }
  }
  recvWithEndMarker();

  if (newData == true) {
    strcpy(tempChars, receivedChars);
    parseData();
    functionCheck();
#ifdef debug
    showParsedData();
#endif
    newData = false;
  }
}


//==================================================================================
void(* resetFunc) (void) = 0;//declare reset function at address 0

//==================================================================================

//==================================================================================
void recvWithEndMarker() {
  static byte ndx = 0;
  char endMarker = '\r'; // \n
  char rc;

  while (Serial.available() > 0 && newData == false) 
  {
    rc = Serial.read();
    //Serial.print(rc);
    if (rc != endMarker) 
    {
      receivedChars[ndx] = rc;
      ndx++;
      if (ndx >= numChars) 
      {
        ndx = numChars - 1;
      }
    }
    else {
      receivedChars[ndx] = '\0'; // terminate the string
      ndx = 0;
      newData = true;
    }
  }
}

//==================================================================================
void parseData() {      // split the data into its parts

  char * strtokIndx; // this is used by strtok() as an index

  strtokIndx = strtok(tempChars, ";");     // get the first part - the string
  strcpy(messageFromPC, strtokIndx); // copy it to messageFromPC

  strtokIndx = strtok(NULL, ";"); // this continues where the previous call left off
  integerFromPC[0] = atoi(strtokIndx);     // convert this part to an integer

  strtokIndx = strtok(NULL, "/");
  integerFromPC[1] = atoi(strtokIndx);    // convert this part to a float


  strtokIndx = strtok(NULL, "\n");
  integerFromPC[2] = atoi(strtokIndx);     // convert this part to a float
}

//==================================================================================
void functionCheck()
{
  if (strcmp(messageFromPC, "SETM")  == 0)
  {
    if (integerFromPC[0] == _ID)
    {
      eeprom_write_action(integerFromPC[1]);
      _ID = integerFromPC[1];
      EEPROM.write(0x03 , 0x00);
      Serial.print("NEW Adress:");
      Serial.println(eeprom_read_action());
      Serial.println("DONE");
    }
  }

  if (strcmp(messageFromPC, "SETP")  == 0)
  {
    EEPROM.write(0x03 , 0xFF);
    Serial.print("First Start done: ID Set to 999");
    resetFunc(); //call reset
  }

  if (strcmp(messageFromPC, "K")  == 0)
  {
    output_reset();
    if (integerFromPC[0] == _ID)
    {
      if ((integerFromPC[1] >= 0) && (integerFromPC[1] <= 10))
      {
        digitalWrite (cathodes[integerFromPC[1]] , HIGH);
#ifdef debug
        Serial.print(cathodes[integerFromPC[1]]);
        Serial.println("set");
#endif
        waittime = true;
        previousMillis = currentMillis;
      }
      else
      {
        Serial.println("UNKNOWN COMMAND in LAGER-Nummer");
      }
      if ((integerFromPC[2] >= 0) && (integerFromPC[2] <= 5))
      {
        digitalWrite (anodes[integerFromPC[2]] , HIGH);
        
#ifdef debug
        Serial.print(anodes[integerFromPC[2]]);
        Serial.println("set");
#endif

      }
      else
      {
        Serial.println("UNKNOWN COMMAND in FACH");
      }
    }
    else
    {
      waittime = false;
      output_reset();
    }
  }
}

//==================================================================================
void showParsedData() {
  Serial.print("My ID ");
  Serial.println(_ID);
  Serial.print("Message ");
  Serial.println(messageFromPC);
  Serial.print("Kasten ");
  Serial.println(integerFromPC[0]);
  Serial.print("Spalte ");
  Serial.println(integerFromPC[1]);
  Serial.print("Fach ");
  Serial.println(integerFromPC[2]);

}

//==================================================================================
void output_reset()
{
  for (int i = 0; i < 12; i++)
  {
    digitalWrite (cathodes[i], LOW);
  }
  for (int i = 0; i < 7; i++)
  {
    digitalWrite (anodes[i], LOW);
  }
}

//==================================================================================
void eeprom_write_action(int IDa)
{
  byte low, high;
  low = IDa & 0xFF;
  high = (IDa >> 8) & 0xFF;
  EEPROM.write(0x01, low); // dauert 3,3ms
  EEPROM.write(0x02, high);
} //eepromWriteInt

//==================================================================================
int eeprom_read_action()
{
  byte low, high;
  low = EEPROM.read(0x01);
  high = EEPROM.read(0x02);
  return low + ((high << 8) & 0xFF00);
} //eepromReadInt

